xquery version "3.1";
(:~
 : This modules handles the automatic mapping of @xml:id to the split files in
 : which they occur.
 :
 : @author Michelle Weidling
 : @version 1.0
 :)
module namespace id-file="https://bdn-edition.de/ns/id-file";

declare namespace bdn="http://www.bdn-edition.de/ns/1.0";
declare namespace tei="http://www.tei-c.org/ns/1.0";

import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace bdn-utils="http://bdn-edition.de/ns/utils" at "utils.xqm";
import module namespace functx="http://www.functx.com";
import module namespace whitelist="https://bdn-edition.de/ns/whitelist" at "whitelist.xqm";
(: import module namespace console="http://exist-db.org/xquery/console"; :)

(:~
 : Creates the file "id-file.xml" for all authors that are currently edited.
 : This is indicated by the fact that a work already has known textgrid URIs.
 :
 : In the ID file we create a mapping of a file that has been the result of the
 : XML splitting process and all the @xml:id that occur in it. This is needed
 : e.g. for construction URLs for the links in the indices HTML sites.
 :
 : @author Michelle Weidling
 : @version 1.0
 : @param $log the location of the author's log file, e.g. "/db/apps/sade/log/noesselt-log.xml"
 : @return xs:string The path where the file is stored ($config:aux)
 :)
declare function id-file:make-id-file($log as xs:string) as xs:string {
    let $add-log-entry := bdn-utils:add-log-entry($log, "Make ID file")

    let $whitelist := whitelist:get-whitelist()
    let $relevant-file-names := id-file:get-relevant-file-names($whitelist)
    let $files-with-ids := id-file:make-file-overview-with-uris($relevant-file-names)
    let $id-file-content := id-file:make-id-file-content($files-with-ids)
    return xmldb:store($config:aux, "id-file.xml", $id-file-content)
};


(:~
 : Returns all split file names that have been transformed to the intermediate
 : format. The ones that still have the original encoding are marked by the
 : prefix "NIF".
 :)
declare function id-file:get-relevant-file-names($whitelist as node()*)
as xs:string* {
    $whitelist//bdn:split-files/bdn:file/string()[not(matches(., "-NIF"))]
};


declare function id-file:make-file-overview-with-uris($relevant-file-names
as xs:string*) as element(bdn:file)* {
    for $file-name in $relevant-file-names
        let $uri := substring-before($file-name, ".xml")
        let $author :=
            if (matches($file-name, "bahrdt-semler")) then
                "bahrdt-semler"
            else
                substring-before($file-name, "-")
        let $xml := doc($config:edition || $author || "/xml/" || $file-name)
        let $ids :=
            for $id in $xml//*[@xml:id]/@xml:id/string() return
                <id xmlns="http://www.bdn-edition.de/ns/1.0">
                    {$id}
                </id>
        return
            <file xmlns="http://www.bdn-edition.de/ns/1.0" uri="{$uri}">
                {$ids}
            </file>
};


(:~
 : A constructor for the ID file elements.
 :)
declare function id-file:make-id-file-content($files-with-ids
as element(bdn:file)*) as element(bdn:id-file) {
    <id-file xmlns="http://www.bdn-edition.de/ns/1.0">
        {$files-with-ids}
    </id-file>
};

(:~
 : Returns the opened ID file.
 :)
declare function id-file:get-id-file() {
  doc($config:aux || "id-file.xml")
};
